import 'dart:async';

import 'package:flutter/services.dart';
import 'package:witsystem_device_plugin/MethodName.dart';
import 'package:witsystem_device_plugin/WitsystemSDK.dart';
import 'package:witsystem_device_plugin/command/CardType.dart';
import 'package:witsystem_device_plugin/command/DeviceModel.dart';
import 'package:witsystem_device_plugin/command/FaceType.dart';
import 'package:witsystem_device_plugin/command/MarkType.dart';

import 'dart:convert' as convert;

import 'package:witsystem_device_plugin/command/PasswordType.dart';
import 'package:witsystem_device_plugin/vibrate/Vibrate.dart';

/// 设备的基本蓝牙操作

class BleDevice extends WisystemSDk {
  final List<StreamSubscription> _bleConnectStateList = []; //蓝牙连接状态的监听者

  final Stream<Map<String, dynamic>> _bleConnectState =
      const EventChannel(ChannelName.bleConnectState)
          .receiveBroadcastStream()
          .map((event) => convert.json.decode(event.toString()));

  factory BleDevice() => _getInstance();

  static BleDevice get instance => _getInstance();
  static BleDevice? _instance;

  BleDevice._internal();

  static BleDevice _getInstance() {
    _instance ??= BleDevice._internal();
    return _instance!;
  }

  /// 连接设备并认证并且可以开启一次设备
  connectAndOpen(String deviceId, bool isOpen, {bool isVibrate = false}) async {
    try {
      var rtn = await methodChannel.invokeMethod("connectAndOpen", {
        "deviceId": deviceId,
        "isOpen": isOpen,
      });
      if (isOpen && isVibrate) Vibrate().start();
      return rtn;
    } catch (e) {
      rethrow;
    }
  }

  /// 当前连接设备的蓝牙连接状态监听
  StreamSubscription bleDeviceConnectState(void Function(dynamic data) event) {
    StreamSubscription streamSubscription = _bleConnectState.listen(event);
    _bleConnectStateList.add(streamSubscription);
    return streamSubscription;
  }

  /// 取消当前连接设备的蓝牙连接状态监听
  /// 当参数为空的时候清空所有监听
  cancelBleDeviceConnectState({StreamSubscription? subscription}) async {
    if (subscription == null) {
      for (var value in _bleConnectStateList) {
        value.cancel();
      }
      _bleConnectStateList.clear();
    } else {
      subscription.cancel();
      _bleConnectStateList.remove(subscription);
    }
  }

  /// 通过蓝牙添加密码
  addPassword(String deviceId, PasswordType passwordType, String password,
      String passwordName,
      {String? userId}) async {
    return await methodChannel.invokeMethod("addPassword", {
      "deviceId": deviceId,
      "passwordType": passwordType.name,
      "password": password,
      "passwordName": passwordName,
      "userId": userId
    });
  }

  /// 通过蓝牙删除密码
  deletePassword(String deviceId, String uid) async {
    return await methodChannel
        .invokeMethod("deletePassword", {"deviceId": deviceId, "uid": uid});
  }

  /// 通过蓝牙添加卡
  /// 等待刷卡回调
  addCard(String deviceId, CardType cardType, String cardName,
      {String? userId, Function()? swingCard}) async {
    methodChannel.setMethodCallHandler((call) {
      if (call.method == "sendSuccess") {
        methodChannel.setMethodCallHandler(null);
        if (swingCard != null) swingCard();
      }
      return Future.value("SUCCESS");
    });

    try {
      var info = await methodChannel.invokeMethod("addCard", {
        "deviceId": deviceId,
        "cardType": cardType.name,
        "cardName": cardName,
        "userId": userId
      });
      return info;
    } catch (e) {
      methodChannel.setMethodCallHandler(null);
      rethrow;
    }
  }

  /// 取消蓝牙添加卡
  cancelAddCard(String deviceId) async {
    return await methodChannel
        .invokeMethod("cancelAddCard", {"deviceId": deviceId});
  }

  /// 通过蓝牙删除卡
  deleteCard(String deviceId, String uid) async {
    return await methodChannel
        .invokeMethod("deleteCard", {"deviceId": deviceId, "uid": uid});
  }

  /// 通过蓝牙冻结或者解冻卡
  freezeCard(String deviceId, String uid, bool isFreeze) async {
    return await methodChannel.invokeMethod(
        "freezeCard", {"deviceId": deviceId, "uid": uid, "isFreeze": isFreeze});
  }

  /// 通过蓝牙开启或者关闭刷卡功能
  closeOrOpenCard(String deviceId, bool isClose) async {
    return await methodChannel.invokeMethod(
        "closeOrOpenCard", {"deviceId": deviceId, "isClose": isClose});
  }

  /// 通过蓝牙添加指纹
  Future addMark(
    String deviceId,
    MarkType markType,
    String markName, {
    String? userId,
    Function()? sendSuccess,
    Function(Map map)? progress,
  }) {
    methodChannel.setMethodCallHandler((call) {
      switch (call.method) {
        case "progress":
          if (progress != null) {
            progress(convert.json.decode(call.arguments));
          }
          break;
        case "sendSuccess":
          if (sendSuccess != null) {
            sendSuccess();
          }
          break;
      }
      return Future.value("SUCCESS");
    });
    try {
      return methodChannel.invokeMethod("addMark", {
        "deviceId": deviceId,
        "markType": markType.name,
        "markName": markName,
        "userId": userId
      });
    } catch (e) {
      methodChannel.setMethodCallHandler(null);
      rethrow;
    }
  }

  /// 取消蓝牙添加指纹
  cancelAddMark(String deviceId) async {
    return await methodChannel
        .invokeMethod("cancelAddMark", {"deviceId": deviceId});
  }

  /// 通过蓝牙删除指纹
  deleteMark(String deviceId, String uid) async {
    return await methodChannel
        .invokeMethod("deleteMark", {"deviceId": deviceId, "uid": uid});
  }

  /// 通过蓝牙冻结或者解冻指纹
  freezeMark(String deviceId, String uid, bool isFreeze) async {
    return await methodChannel.invokeMethod(
        "freezeMark", {"deviceId": deviceId, "uid": uid, "isFreeze": isFreeze});
  }

  /// 通过蓝牙添加人脸
  addFace(String deviceId, FaceType faceType, String faceName,
      {String? userId}) async {
    return await methodChannel.invokeMethod("addFace", {
      "deviceId": deviceId,
      "faceType": faceType.name,
      "faceName": faceName,
      "userId": userId
    });
  }

  /// 通过蓝牙添加人脸
  cancelAddFace(String deviceId) async {
    return await methodChannel
        .invokeMethod("cancelAddFace", {"deviceId": deviceId});
  }

  /// 通过蓝牙删除人脸
  deleteFace(String deviceId, String uid) async {
    return await methodChannel
        .invokeMethod("deleteFace", {"deviceId": deviceId, "uid": uid});
  }

  /// 通过蓝牙冻结或者解冻人脸
  freezeFace(String deviceId, String uid, bool isFreeze) async {
    return await methodChannel.invokeMethod(
        "freezeFace", {"deviceId": deviceId, "uid": uid, "isFreeze": isFreeze});
  }

  /// 通过蓝牙修改音量
  updateVolume(String deviceId, int volume) async {
    return await methodChannel
        .invokeMethod("updateVolume", {"deviceId": deviceId, "volume": volume});
  }

  /// 设置设备模式
  setModel(String deviceId, DeviceModel model) async {
    return await methodChannel
        .invokeMethod("setModel", {"deviceId": deviceId, "model": model.name});
  }

  /// 通过蓝牙更新设备状态
  updateDeviceState(String deviceId) async {
    return await methodChannel
        .invokeMethod("updateDeviceState", {"deviceId": deviceId});
  }

  /// 通过关闭指定设备的蓝牙操作
  closeBleDevice(String deviceId) async {
    return await methodChannel
        .invokeMethod("closeBleDevice", {"deviceId": deviceId});
  }
}
